package test;

/*DynamicArrayTest.java

Test code that tests the DynamicArray class

EDIT INSTRUCTIONS:

	1. Code tests -- add/remove/edit the example tests with
		your test code (your test code may follow the examples closely)

	2. In "run" method, edit tests as appropriate so they call tests
		coded in step #1.

*/

import dynarray.DynamicArray;

public class DynamicArrayTest {

	//---------------------------------------
	// Public

	public static void main(String[] args) {
		(new DynamicArrayTest()).run();
	}

	public void run() {
		//run experiments
		this.println("");

		this.basicRun("smokeTest_construct_newEmpty", () -> smokeTest_construct_newEmpty());
		this.basicRun("smokeTest_construct_fromCapacityGrowthFactor", () -> smokeTest_construct_fromCapacityGrowthFactor());
		
		this.basicRun("smokeTest_size", () -> smokeTest_size());		
		this.basicRun("smokeTest_add", () -> smokeTest_add());
		this.basicRun("smokeTest_get", () -> smokeTest_get());
		
		this.basicRun("smokeTest_load", () -> smokeTest_load());		
		this.basicRun("smokeTest_object_load", () -> smokeTest_object_load());		
						
	}

	//---------------------------------------
	//Constructor Smoke Tests

	private void smokeTest_construct_newEmpty() {
		//We will simply test that the constructors run without crashing (without throwing an exception)
		@SuppressWarnings("unused")
		DynamicArray<Employee> list = DynamicArray.newEmpty();
		//If we get here, constructor did not "crash", so we pass
		this.println("PASSED");
	}

	private void smokeTest_construct_fromCapacityGrowthFactor() {
		//We will simply test that the constructors run without crashing (without throwing an exception)
		@SuppressWarnings("unused")
		DynamicArray<Employee> list = DynamicArray.fromCapacityGrowthFactor(20, 3);
		//If we get here, constructor did not "crash", so we pass
		this.println("PASSED");
	}
	
	//---------------------------------------
	//ADT Core Method Smoke Tests
	
	public void smokeTest_size() {
		DynamicArray<String> dynArray = DynamicArray.newEmpty();
		printf("Size (expecting 0): %d%n", dynArray.size());
		this.assertEquals(0, dynArray.size());
	}

	public void smokeTest_add() {
		DynamicArray<String> dynArray = DynamicArray.newEmpty();
		dynArray.add("asha@hello.com");
		printf("Size (expecting 1): %d%n", dynArray.size());
		this.assertEquals(1, dynArray.size());		
	}

	public void smokeTest_get() {
		DynamicArray<String> dynArray = DynamicArray.newEmpty();
		String email = "asha@hello.com";
		dynArray.add(email);
		printf("First Element (expecting [%s]): %s%n", email, dynArray.get(0));
		this.assertEquals(email, dynArray.get(0));		
	}
	
	//---------------------------------------
	//ADT Core Method Load Tests
	
	public void smokeTest_load() {
		DynamicArray<String> dynArray = DynamicArray.newEmpty();
		for (int i = 0; i < 1000; i++)
			dynArray.add("Asha" + i);
		for (int i = 0; i < 1000; i += 50)
			printf("Element at [%d]: %s%n", i, dynArray.get(i));
	}

	public void smokeTest_object_load() {
		DynamicArray<Employee> dynArray = DynamicArray.newEmpty();
		for (int i = 0; i < 5000; i++)
			dynArray.add(new Employee(i, "Name" + i));
		for (int i = 0; i < 5000; i += 250)
			printf("Element at index [%d]: %s%n", i, dynArray.get(i));
	}	

	//---------------------------------------
	// Helpers

	private void basicRun(String methodName, BlockClosure testMethod) {
		//we bind so the method is associated with this object
		this.printf("Starting - %s%n", methodName);
		try {
			testMethod.evaluate();
		} catch(Exception ex) {
			this.printf("EXCEPTION OCCURRED in %s -- %s%n", methodName, ex.getMessage());
		}
		this.printf("Finished - %s%n%n", methodName);
	}
	
	protected void assertEquals(Object expected, Object actual) {
		String msg = actual.equals(expected)
						? "PASSED"
						: String.format("FAILED -- expected=%s -- actual=%s", expected, actual);
		System.out.println(msg);
	}	

	protected void println(Object o) {
		System.out.println(o.toString());
	}

	protected void printf(String template, Object ...args) {
		System.out.printf(template, args);
	}

}

//---------------------------------------------------------
//Helper Interfaces and Classes

@FunctionalInterface
interface BlockClosure {
    void evaluate();
}


